@@ -1,20 +1,18 @@
-# TODO: Incorporate all the monkey patching in here!
-
 # TODO: What is this
 from __future__ import generators
 
-import decimal, re, inspect
-import copy
-
+import decimal, inspect, StringIO
 from django.db.models.query import QuerySet
-from django.db.models import Model, permalink
+from django.db.models import Model
 from django.utils import simplejson
 from django.utils.xmlutils import SimplerXMLGenerator
 from django.utils.encoding import smart_unicode
-from django.core.urlresolvers import reverse, NoReverseMatch
 from django.core.serializers.json import DateTimeAwareJSONEncoder
-from django.http import HttpResponse
-from django.core import serializers
+
+# Class which will register MimeTypes to methods which will decode the
+# corresponding MimeType to python data structures.
+from utils import Mimer
+from handlers import ModelHandler
 
 try:
 	import xlwt
@@ -22,20 +20,8 @@ except ImportError:
 	# Needed for sphinx documentation
 	# WTF
 	pass
-import StringIO
 
-# Class which will register MimeTypes to methods which will decode the
-# corresponding MimeType to python data structures.
-from utils import Mimer
 
-#TODO: Needed?
-#from validate_jsonp import is_valid_jsonp_callback_value
-
-#TODO: WTF?
-# Allow people to change the reverser (default `permalink`).
-reverser = permalink
-
-from handlers import BaseHandler, ModelHandler
 
 class Emitter:
     """
@@ -176,7 +162,6 @@ class Emitter:
                 # TODO: If the request does not ask for nested representation,
                 # then give resource URI instead.
                 fields = set(handler.allowed_out_fields) - set(handler.exclude_nested)
-
             
             if handler:
                 get_fields = set(fields)
@@ -338,17 +323,10 @@ class JSONEmitter(Emitter):
     JSON emitter, understands timestamps.
     """
     def render(self, request):
-        # TODO: Get rid of this callback shit
-        cb = request.GET.get('callback', None)
         data_as_dic = self.construct()
         seria = simplejson.dumps(data_as_dic, cls=DateTimeAwareJSONEncoder, ensure_ascii=False, indent=4)
 
-        # Callback
-        if cb and is_valid_jsonp_callback_value(cb):
-            return '%s(%s)' % (cb, seria)
-
         return seria
-
 Emitter.register('json', JSONEmitter, 'application/json; charset=utf-8')
 Mimer.register(simplejson.loads, ('application/json',))
 
@@ -456,23 +434,27 @@ class ExcelEmitter(Emitter):
     # TODO
     # Works only for outputting handlers extending the ModelHandler class
     # Doesn't really work with outputing nested fields 
-
 Emitter.register('excel', ExcelEmitter, 'application/vnd.ms-excel')
  
                
 class HTMLEmitter(Emitter):
     def render(self, request):
         construct = self.construct()
-
         if 'data' in construct:
-            # Correct response
-            return construct['data']
+            # If the ``construct['data']`` is dictionary(hence the request was
+            # singular), and only contains one field,
+            # we can simmly output the  ``value`` of this field. 
+            # Else, we simply output the whole data structure 
+            # as is (altough it doesn't make much sense).
+            if isinstance(construct['data'], dict)\
+            and len(construct['data']) == 1:
+                return construct['data'].values()[0]
+            else:
+                return construct['data']
 
         elif 'errors' in construct:
             # Validation was raised
             return construct['errors']
 
         return None         
-        
-
 Emitter.register('html', HTMLEmitter, 'text/html')
