@@ -2,7 +2,7 @@ from django.db import models
 from authentication import DjangoAuthentication, NoAuthentication
 from django.core.exceptions import ValidationError, ObjectDoesNotExist
 from custom_filters import filter_to_method
-from exceptions import UnprocessableEntity
+from exceptions import UnprocessableEntity, ValidationErrorList
 
 # mappings of {HTTP Request: API Handler method}
 CALLMAP = {
@@ -87,7 +87,6 @@ class BaseHandler():
     don't hesitate to override.
     """
     __metaclass__ = BaseHandlerMeta
-            
 
     allowed_out_fields = ()
     """
@@ -135,7 +134,7 @@ class BaseHandler():
     bulk_create = False
     """
     Indicates whether bulk POST(creating multiple items with one request) requests are allowed.
-    """
+    """                
 
     plural_update = False
     """
@@ -349,9 +348,9 @@ class BaseHandler():
                 # In the case of PUT requests, we first force the evaluation of
                 # the affected dataset (theferore if there are any
                 # HttpResourceGone exceptions, they will be raised now), and then in the
-                # ``validate`` method, we perform any data validations.
-                dataset = self.data(request, *args, **kwargs)              
-                kwargs['dataset'] = dataset
+                # ``validate`` method, we perform any data validations. We
+                # assign it to parameter ``request.dataset``.
+                request.dataset = self.data(request, *args, **kwargs)              
             self.validate(request, *args, **kwargs)
         
         # Pick action to run
@@ -486,7 +485,7 @@ class ModelHandler(BaseHandler):
     value given to ``id``. So, the querystring ``?id=12&id=14``, will perform
     the filter ``filter(id__in=[12, 14]``), on the corresponding model.
     """
-
+    
     read = True
     create = True
     update = True
@@ -504,17 +503,62 @@ class ModelHandler(BaseHandler):
         
         After this method, we shouldn't perform modifications on the model
         instances, since any modifications might make the data models invalid.
+
+        For Bulk create or plural update requests, if some of the data
+        instances fail to validate, we raise an ErrorList exception, which
+        includes the validation errors of the corresponding data instances.
         """
+        def validate_all_post(instances):
+            """
+            Generator that validates a list of ``self.model`` instances.
+            Should be used to validate the model instance in the case of Bulk
+            POST and plural PUT requests.
+
+            @param instances: List of ``self.model`` instances
+            @param specifier: String to indicate the position of erroneous
+            instances.
+
+            For every instance in ``instances``, it returns None if the
+            instance validates correctly, or a ValidationError if not.
+            """
+            for i, instance in enumerate(instances):
+                try:
+                    instance.full_clean()
+                except ObjectDoesNotExist:
+                    yield ValidationError(
+                        'Foreign Keys on model not defined',
+                        params={'index': i}
+                    )
+                except ValidationError, e:
+                    e.params = {'index': i}
+                    yield e
+                yield None
+
+        def validate_all_put(instances): 
+            for instance in instances:
+                try:
+                    instance.full_clean()
+                except ObjectDoesNotExist:
+                    yield ValidationError(
+                        'Foreign Keys on model not defined',
+                        params={'id': instance.id}
+                    )
+                except ValidationError, e:
+                    e.params = {'id': instance.id}
+                    yield e
+                yield None
+
         super(ModelHandler, self).validate(request, *args, **kwargs)
 
         if request.method.upper() == 'POST':
-            # Create model instance(s) without saving them, and go through all
-            # field level validation checks.
+            # Create model instance(s) (without saving them), and validate them
             if not isinstance(request.data, list):
+                # Single model instance. 
+                #Validate and raise any exception that may happen                
                 request.data = self.model(**request.data)
                 try:                    
                     request.data.full_clean()                    
-                except ObjectDoesNotExist, e:
+                except ObjectDoesNotExist:
                     # There is a weird case, when if a Foreign Key on the model
                     # instance is not defined, and this foreign key is used in
                     # the __unicode__ method of the model, to derive its string
@@ -525,32 +569,44 @@ class ModelHandler(BaseHandler):
                     # from the model's unicode method, but that would require a
                     # lot of manual work
                     raise ValidationError('Foreign Keys on model not defined')
+                except ValidationError:
+                    raise
+            else:                   
+                # Multiple model instances. 
+                # Create(not save), validate all, make a list of all exceptions
+                # that may happen, and pack them in a ValidationErrorList.
+                request.data = \
+                    [self.model(**data_item) for data_item in request.data]
+                
+                error_list = [error for error in validate_all_post(request.data) if error]
+                if error_list:
+                    raise ValidationErrorList(error_list)
 
-            else:
-                request.data = [self.model(**data_item) for \
-                    data_item in request.data]
-                for instance in request.data:
-                    try:
-                        instance.full_clean()
-                    except ObjectDoesNotExist, e:
-                        raise ValidationError('Foreign Keys on model not defined')
-
-        elif request.method.upper() == 'PUT':      
-            current = kwargs.pop('dataset', None)  # Evaluated in ``execute_request``        
-
-            def update(current, data):
-                update_values = data.items()
-                for instance in isinstance(current, self.model) and [current]\
-                    or current:
-                    for field, value in update_values:
-                        setattr(instance, field, value)
-                    instance.full_clean()   
-            # Update all model instances in ``current``, with the data from the
-            # reqeust body.
-            update(current, request.data)
+        elif request.method.upper() == 'PUT':     
+            current = getattr(request, 'dataset', None)
             
+            def update(instance, update_items):
+                # Update ``instance`` with the key, value pairs in
+                # ``update_values``
+                [setattr(instance, field, value) for field, value in update_items]
+
+            # (key, value) pairs to update                
+            update_items = request.data.items()
+
+            if isinstance(current, self.model):
+                # Single model instance
+                # Update, validate and raise any exception that may happen
+                update(current, update_items)
+                current.full_clean()
+            else:
+                # Multiple model instance
+                # Update, validate all, make list of all exceptions that
+                # occur, and pack them in a ValidationErrorList.
+                [update(instance, update_items) for instance in current]
+                error_list = [error for error in validate_all_put(current) if error]
+                if error_list:
+                    raise ValidationErrorList(error_list)
             request.data = current
-  
 
     def working_set(self, request, *args, **kwargs):
         """
@@ -671,30 +727,47 @@ class ModelHandler(BaseHandler):
 
     def create(self, request, *args, **kwargs):
         """
-        Saves the model instances available in ``request.data``. Returns the
-        subset of ``request.data`` which contains the successfully created
-        model instances.
-        """
-        if not isinstance(request.data, self.model):
-            unsuccessful = []
-            for instance in request.data:
-                try:
-                    instance.save(force_insert=True)
-                except:
-                    unsuccessful.append(instance)
-            if unsuccessful:
-                # Remove model instances that were not saved successfully, from
-                # ``request.data``
-                request.data = set(request.data) - set(unsuccessful)
-        else:
-            # request.data is a single self.model instance
-            try:
-                request.data.save(force_insert=True)
-            except:
-                # Not sure what errors we could get, but I think it's safe to just
-                # assume that *any* error means that no record has been created.
-                request.data = None
+        Saves the model instances available in ``request.data``. 
         
+        After this method, ``request.data`` only contains the successfully
+        created model instance(s).
+
+        When can a model instance fail to be created?
+         * In very rare cases, when a model instance will escape the uniqueness
+           constraints(eg, bulk create: More that one entries are the same.
+           They both escape the uniqueness constraints since they are not yet
+           created, but only the first of them managed to be created eventually), 
+           and only fail upon hitting the database.
+         * Database failure
+        """
+        def persist(instance):
+            try:
+                instance.save(force_insert=True)
+            except:
+                # TODO: 
+                # 1. If I was using InnoDB storage engine, I could simply consider
+                # the whole operation (whether single or Bulk) as a
+                # transaction, and roll it back in the first failure.
+                # Unfortunately I cannot assume that InnoDB is used
+                # 
+                # 2. Use bulk_create that Django 1.4 offers
+                #
+                # Long term plan:
+                # - Single item POST: Raise an error and an appropriate
+                #   HTTPResponse, when the database fails.
+                # - Bulk POST: Use Django 1.4 ``bulk_create``. This way a
+                # failure can raise an error.
+                #
+                return None
+            else:
+                return instance
+
+        if isinstance(request.data, self.model):
+            request.data = persist(request.data)
+        elif request.data:
+            request.data = \
+                [instance for instance in request.data if persist(instance)]
+
         return super(ModelHandler, self).create(request, *args, **kwargs)
     
     def update(self, request, *args, **kwargs):
@@ -706,14 +779,16 @@ class ModelHandler(BaseHandler):
         def persist(instance):
             try:
                 instance.save(force_update=True)
-                return instance
             except:
                 return None
+            else:
+                return instance
         
         if isinstance(request.data, self.model):
             request.data = persist(request.data)
         elif request.data:
-            request.data = [instance for instance in request.data if persist(instance)]
+            request.data = \
+                [instance for instance in request.data if persist(instance)]
         
         return super(ModelHandler, self).update(request, *args, **kwargs)
     
