@@ -1,24 +1,22 @@
 from django.views.decorators.vary import vary_on_headers
 from django.http import HttpResponse
-from authentication import NoAuthentication
 from django.conf import settings
 
 from django.core.exceptions import ValidationError, ObjectDoesNotExist, \
     PermissionDenied
-from django.http import Http404, HttpResponseBadRequest, \
-    HttpResponseGone, HttpResponseNotAllowed, HttpResponseNotFound, \
+from django.http import HttpResponseBadRequest, \
+    HttpResponseGone, HttpResponseNotAllowed, \
     HttpResponseForbidden, HttpResponseServerError
 
-from django.conf import settings
 from django.db import connection
-import datetime
         
 from utils import coerce_put_post, translate_mime
-from exceptions import MethodNotAllowed, UnprocessableEntity
+from exceptions import MethodNotAllowed, UnprocessableEntity,\
+    ValidationErrorList, UnprocessableEntityList
 from emitters import Emitter, JSONEmitter
                    
 from django.views.debug import ExceptionReporter   
-from django.core.mail import send_mail, EmailMessage
+from django.core.mail import EmailMessage
 import sys
 
 class Resource:
@@ -341,27 +339,40 @@ class Resource:
             return u'django-icetea crash report:\n\n%s' % error
         
         http_response, message = None, ''
-        
-        if isinstance(e, ValidationError):
+
+        def validation_error_message(e):
             if hasattr(e, 'message_dict'):
                 if '__all__' in e.message_dict:
-                    # In the case of a ``clean`` model method, that raises a
-                    # ValidationError with a string argument.           
                     errors = e.message_dict['__all__']
                 else:
-                    # In the case of a ``clean`` model method, that raises a
-                    # ValidationError with a dict argument.                    
                     errors = e.message_dict
             elif hasattr(e, 'messages'):
-                # Generic ValidationError messages
                 errors = e.messages
-
             message = dict(
                 type='Validation Error',
                 errors=errors,
             )
+            if hasattr(e, 'params') and e.params:
+                message.update(**e.params)
+            return message
+
+        def unprocessable_entity_message(e):
+            message = dict(
+                type='Unprocessable Entity Error',
+                errors=e.errors,
+            )
+            if hasattr(e, 'params') and e.params:
+                message.update(**e.params)
+            return message
+        
+        if isinstance(e, ValidationError):
+            message = validation_error_message(e)
             http_response = HttpResponseBadRequest()
 
+        elif isinstance(e, UnprocessableEntity):
+            message = unprocessable_entity_message(e)
+            http_response = HttpResponse(status=422)
+
         elif isinstance(e, (NotImplementedError, ObjectDoesNotExist)):
             http_response = HttpResponseGone()
 
@@ -371,12 +382,15 @@ class Resource:
         elif isinstance(e, PermissionDenied):
             http_response = HttpResponseForbidden()
 
-        elif isinstance(e, UnprocessableEntity):
-            message = dict(
-                type='Unprocessable Entity Error',
-                errors=e.errors,
-            )
+        elif isinstance(e, ValidationErrorList):       
+            http_response = HttpResponseBadRequest()
+            # TODO: Differentiate between validation error and unprocessable
+            # entity errors
+            message = [validation_error_message(error) for error in e.error_list]
+
+        elif isinstance(e, UnprocessableEntityList):
             http_response = HttpResponse(status=422)
+            message = [unprocessable_entity_message(error) for error in e.error_list]
 
         else: 
             # Consider it a Server Error.
@@ -418,8 +432,13 @@ class Resource:
         extend with custom debug information.
         """
         if settings.DEBUG:
+            total_query_time = sum(
+                [float(dic['time']) for dic in connection.queries]
+             )
+            
             response_dictionary.update({
                 'debug': {
+                    'total_query_time': total_query_time,
                     'query_count': len(connection.queries),
                     'query_log': connection.queries,
                 }
